@@ -32,10 +32,17 @@ from   pyflyby._modules         import ModuleHandle
 from   pyflyby._parse           import PythonBlock, infer_compile_mode
 
 
+NoneType = type(None)
+EllipsisType = type(Ellipsis)
+
 class _ClassScope(dict):
     pass
 
 
+    def __repr__(self):
+        return "_ClassScope(" + repr(super()) + ")"
+
+
 _builtins2 = {"__file__": None}
 
 
@@ -52,7 +59,7 @@ class ScopeStack(Sequence):
 
     _cached_has_star_import = False
 
-    def __init__(self, arg):
+    def __init__(self, arg, _class_delayed=None):
         """
         Interpret argument as a ``ScopeStack``.
 
@@ -90,6 +97,22 @@ class ScopeStack(Sequence):
         tup = tuple(result)
         self._tup = tup
 
+        # class name definitions scope may need to be delayed.
+        # so we store them separately, and if they are present in methods def, we can readd them
+        if _class_delayed is None:
+            _class_delayed = {}
+        self._class_delayed = _class_delayed
+
+    def __contains__(self, item):
+        if isinstance(item, DottedIdentifier):
+            item = item.name
+        if isinstance(item, str):
+            for sub in self:
+                if item in sub.keys():
+                    return True
+
+        return False
+
     def __getitem__(self, item):
         if isinstance(item, slice):
             return self.__class__(self._tup[item])
@@ -98,7 +121,9 @@ class ScopeStack(Sequence):
     def __len__(self):
         return len(self._tup)
 
-    def with_new_scope(self, include_class_scopes=False, new_class_scope=False):
+    def with_new_scope(
+        self, include_class_scopes=False, new_class_scope=False, unhide_classdef=False
+    ):
         """
         Return a new ``ScopeStack`` with an additional empty scope.
 
@@ -106,6 +131,8 @@ class ScopeStack(Sequence):
           Whether to include previous scopes that are meant for ClassDefs.
         :param new_class_scope:
           Whether the new scope is for a ClassDef.
+        :param unhide_classdef:
+          Unhide class definitiion scope (when we enter a method)
         :rtype:
           ``ScopeStack``
         """
@@ -119,7 +146,9 @@ class ScopeStack(Sequence):
         else:
             new_scope = {}
         cls = type(self)
-        result = cls(scopes + (new_scope,))
+        if unhide_classdef and self._class_delayed:
+            scopes = tuple([self._class_delayed]) + scopes
+        result = cls(scopes + (new_scope,), _class_delayed=self._class_delayed)
         return result
 
     def clone_top(self):
@@ -178,6 +207,21 @@ class ScopeStack(Sequence):
             # there might be a star import later.
             return False
 
+    def __repr__(self):
+        scopes_reprs = [
+            "{:2}".format(i) + " : " + repr(namespace)
+            for i, namespace in enumerate(self)
+        ][1:]
+
+        return (
+            "<{class_name} object at 0x{hex_id} with namespaces: [\n".format(
+                class_name=self.__class__.__name__, hex_id=id(self)
+            )
+            + " 0 : {builtins namespace elided.}\n"
+            + "\n".join(scopes_reprs)
+            + "\n]>"
+        )
+
 
 def symbol_needs_import(fullname, namespaces):
     """
@@ -280,7 +324,11 @@ def symbol_needs_import(fullname, namespaces):
                 return False
     # We didn't find any scope that defined the name.  Therefore it needs
     # import.
-    logger.debug("symbol_needs_import(%r): no match found in namespaces; it needs import", fullname)
+    logger.debug(
+        "symbol_needs_import(%r): no match found in namespaces %s; it needs import",
+        fullname,
+        namespaces,
+    )
     return True
 
 
@@ -345,6 +393,7 @@ class _MissingImportFinder(object):
         self._in_FunctionDef = False
         # Current lineno.
         self._lineno = None
+        self._in_class_def = 0
 
     def find_missing_imports(self, node):
         self._scan_node(node)
@@ -458,7 +507,8 @@ class _MissingImportFinder(object):
                         "unexpected %s" %
                         (', '.join(type(v).__name__ for v in value)))
             elif isinstance(value, (six.integer_types, float, complex,
-                                    str, six.text_type, type(None), bytes)):
+                                    str, six.text_type, NoneType, bytes,
+                                    EllipsisType)):
                 pass
             else:
                 raise TypeError(
@@ -516,21 +566,45 @@ class _MissingImportFinder(object):
         # therefore think that the 'import foo' on L1 could be removed.
         self.visit(node.value)
         self.visit(node.targets)
+        self._visit__all__(node)
 
+    def _visit__all__(self, node):
+        if self._in_FunctionDef:
+            return
+        if (len(node.targets) == 1 and isinstance(node.targets[0], ast.Name)
+            and node.targets[0].id == '__all__'):
+            if not isinstance(node.value, ast.List):
+                logger.warning("Don't know how to handle __all__ as (%s)" % node.value)
+                return
+            if not all(isinstance(e, ast.Str) for e in node.value.elts):
+                logger.warning("Don't know how to handle __all__ with list elements other than str")
+                return
+            for e in node.value.elts:
+                self._visit_Load_defered_global(e.s)
 
     def visit_ClassDef(self, node):
+        logger.debug("visit_ClassDef(%r)", node)
         if PY3:
             assert node._fields == ('name', 'bases', 'keywords', 'body', 'decorator_list')
         else:
             assert node._fields == ('name', 'bases', 'body', 'decorator_list')
         self.visit(node.bases)
         self.visit(node.decorator_list)
+        # The class's name is only visible to others (not to the body to the
+        # class), but is accessible in the methods themselves. See https://github.com/deshaw/pyflyby/issues/147
         if PY3:
             self.visit(node.keywords)
+
+        # we only care about the first defined class,
+        # we don't detect issues with nested classes.
+        if self._in_class_def == 0:
+            self.scopestack._class_delayed[node.name] = None
         with self._NewScopeCtx(new_class_scope=True):
+            if not self._in_class_def:
+                self._in_class_def += 1
+                self._visit_Store(node.name)
             self.visit(node.body)
-        # The class's name is only visible to others (not to the body to the
-        # class).
+            self._in_class_def -= 1
         self._visit_Store(node.name)
 
     def visit_AsyncFunctionDef(self, node):
@@ -560,7 +634,9 @@ class _MissingImportFinder(object):
                 self._visit_typecomment(node.type_comment)
             old_in_FunctionDef = self._in_FunctionDef
             self._in_FunctionDef = True
-            with self._NewScopeCtx():
+            with self._NewScopeCtx(unhide_classdef=True):
+                if not self._in_class_def:
+                    self._visit_Store(node.name)
                 self.visit(node.body)
             self._in_FunctionDef = old_in_FunctionDef
         self._visit_Store(node.name)
@@ -577,9 +653,33 @@ class _MissingImportFinder(object):
             self._in_FunctionDef = old_in_FunctionDef
 
     def _visit_typecomment(self, typecomment):
+        """
+        Warning, when a type comment the node is a string, not an ast node.
+        We also get two types of type comments:
+
+
+        The signature one just after a function definition
+
+            def foo(a):
+                # type: int -> None
+                pass
+
+        And the variable annotation ones:
+
+            def foo(a #type: int
+                ):
+                pass
+
+        ast parse  "func_type" mode only support the first one.
+
+        """
         if typecomment is None:
             return
-        node = ast.parse(typecomment)
+        if '->' in typecomment:
+            node = ast.parse(typecomment, mode='func_type')
+        else:
+            node = ast.parse(typecomment)
+
         self.visit(node)
 
     def visit_arguments(self, node):
@@ -611,9 +711,15 @@ class _MissingImportFinder(object):
             self.visit(node.kwonlyargs)
         if sys.version_info >= (3, 8):
             self.visit(node.posonlyargs)
-        # Store vararg/kwarg names.
-        self._visit_Store(node.vararg)
-        self._visit_Store(node.kwarg)
+        # may be None.
+        if node.vararg and PY3:
+            self.visit(node.vararg)
+        else:
+            self._visit_Store(node.vararg)
+        if node.kwarg and PY3:
+            self.visit(node.kwarg)
+        else:
+            self._visit_Store(node.kwarg)
 
     def visit_ExceptHandler(self, node):
         assert node._fields == ('type', 'name', 'body')
@@ -831,9 +937,32 @@ class _MissingImportFinder(object):
         # Don't call generic_visit(node) here.  Reason: We already visit the
         # parts above, if relevant.
 
+    def _visit_Load_defered_global(self, fullname):
+        """
+        Some things will be resolved in global scope later.
+        """
+        logger.debug("_visit_Load_defered_global(%r)", fullname)
+        if symbol_needs_import(fullname, self.scopestack):
+            data = (fullname, self.scopestack, self._lineno)
+            self._deferred_load_checks.append(data)
+
+
+    def _visit_Load_defered(self, fullname):
+        logger.debug("_visit_Load_defered(%r)", fullname)
+        if symbol_needs_import(fullname, self.scopestack):
+            data = (fullname, self.scopestack.clone_top(), self._lineno)
+            self._deferred_load_checks.append(data)
+
+    def _visit_Load_immediate(self, fullname):
+        logger.debug("_visit_Load_immediate(%r)", fullname)
+        self._check_load(fullname, self.scopestack, self._lineno)
+
+
+
     def _visit_Load(self, fullname):
         logger.debug("_visit_Load(%r)", fullname)
         if self._in_FunctionDef:
+            self._visit_Load_defered(fullname)
             # We're in a FunctionDef.  We need to defer checking whether this
             # references undefined names.  The reason is that globals (or
             # stores in a parent function scope) may be stored later.
@@ -854,24 +983,28 @@ class _MissingImportFinder(object):
             # On the other hand, we intentionally alias the other scopes
             # rather than cloning them, because the point is to allow them to
             # be modified until we do the check at the end.
+            self._visit_Load_defered(fullname)
 
-            if symbol_needs_import(fullname, self.scopestack):
-                data = (fullname, self.scopestack.clone_top(), self._lineno)
-                self._deferred_load_checks.append(data)
         else:
             # We're not in a FunctionDef.  Deferring would give us the same
             # result; we do the check now to avoid the overhead of cloning the
             # stack.
-            self._check_load(fullname, self.scopestack, self._lineno)
+            self._visit_Load_immediate(fullname)
 
     def _check_load(self, fullname, scopestack, lineno):
-        # Check if the symbol needs import.  (As a side effect, if the object
-        # is a _UseChecker, this will mark it as used.  TODO: It would be
-        # better to refactor symbol_needs_import so that it just returns the
-        # object it found, and we mark it as used here.)
+        """
+        Check if the symbol needs import.  (As a side effect, if the object
+        is a _UseChecker, this will mark it as used.
+
+        TODO: It would be
+        better to refactor symbol_needs_import so that it just returns the
+        object it found, and we mark it as used here.)
+        """
+
         fullname = DottedIdentifier(fullname)
         if symbol_needs_import(fullname, scopestack) and not scopestack.has_star_import():
-            self.missing_imports.append((lineno,fullname))
+            if (lineno, fullname) not in self.missing_imports:
+                self.missing_imports.append((lineno, fullname))
 
     def _finish_deferred_load_checks(self):
         for fullname, scopestack, lineno in self._deferred_load_checks:
