@@ -277,6 +277,22 @@ def test_find_missing_imports_function_defaults_kwargs_1():
     result   = _dilist2strlist(result)
     expected = ['args', 'kwargs', 'y']
     assert expected == result
+    
+@pytest.mark.skipif(
+    PY2,
+    reason="Python 3-only syntax.")
+def test_find_missing_imports_kwarg_annotate():
+    """
+    pfb issue 162
+    """
+    code = dedent("""
+        def func_pfb162(args:Dict, **kwargs:Any):
+            pass
+    """)
+    result   = find_missing_imports(code, [{}])
+    result   = _dilist2strlist(result)
+    expected = ['Any', 'Dict']
+    assert expected == result
 
 
 def test_find_missing_imports_function_defaults_kwargs_2():
@@ -1693,6 +1709,22 @@ def test_all_exports_1():
     assert unused == [(2, Import('from os import read'))]
 
 
+def test_all_exports_2():
+    code = dedent("""
+        from __future__ import absolute_import, division
+
+        __all__ = ["defaultdict", "rmdir"]
+
+
+        def defaultdict():
+            pass
+    """)
+    missing, unused = scan_for_import_issues(code)
+    assert missing == [(4, DottedIdentifier('rmdir'))]
+    assert unused == []
+
+
+
 def test_load_symbol_1():
     assert load_symbol("os.path.join", {"os": os}) is os.path.join
 
